/*	libmarty.h - M.A.R-T>Y. C lib helper function header file
	Copyright (C) 2023-2024 hoff.industries

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <https://www.gnu.org/licenses/>. */

#include <stdint.h>

#include <poll.h>
#include <pthread.h>

#ifndef LIBMMP_AUTHOR

#include "libmmp.h"

#endif

#define LIBMARTY_AUTHOR			"hoff.industries"

#define CONN_UNIX				0
#define CONN_TCP				1

#define LOG_NOMINAL				0
#define LOG_INFO				1
#define LOG_WARNING				2
#define LOG_ERROR				3
#define LOG_CRITICAL			4
#define LOG_STARTUP				5
#define LOG_NOTITLE				6
#define LOG_SUCCESS				7

#define MAX_MESSAGE_ID			4096
#define MAX_MESSAGE_ID_LEN		5

#define MAX_CLIENT_ID			4096
#define MAX_CLIENT_ID_LEN		5

#define RECONNECT_INTERVAL		2

#define DEFAULT_PORT			9341

#define EXIT_ERR_CDATA			1
#define EXIT_ERR_VSTRING		2
#define EXIT_ERR_HOSTNAME		3

#define EXIT_ERR_SPAWN			4
#define EXIT_ERR_INIT			5

#define EXIT_ERR_POLL			6
#define EXIT_ERR_ITC			7

#define EXIT_ERR_FORMAT			8
#define EXIT_ERR_CHILD			9
#define EXIT_ERR_SIGN			10
#define EXIT_ERR_READ			11
#define EXIT_ERR_BAD_POINTER	12

#define C_DAEMON_COLOUR			3

#define PAYL_IDX_ENDPOINT		4

#define ITC_DELIVERY_PIPE		0
#define ITC_MESSAGE_PIPE		1

extern const int LIBMARTY_VERSION[3];

const extern char *LoggerColours[];
const extern char MartyColour[];
const extern char ClearColour[];

const extern char MartyFlavour[];
const extern char BracketFlavour[];

const extern int FlavourLen;
const extern int BracketFlavourLen;

struct MartyClientParameters {
	char *auth;
	uint16_t priority;
	char *host;
	char *name;
};

struct MartyMessageHandle {
	uint16_t result;
	struct MMPacket *execute;
	struct MartyMessageHandle *next;
};

struct MartyClientEndpoint {
	int endpoint;
	char *name;

	int min;
	int max;

	bool isAscii;
};

struct MartyClientData {
	char *versionString;

	char *targetHost;
	int targetPort;
	char *targetSocket;

	int connType;
	int logLevel;

	int socket;
	pthread_mutex_t networkLock;

	int nfds;
	struct pollfd pfds[2];

	int netMessagePipe[2];
	int netDeliveryPipe[2];

	struct MartyClientParameters initParameters;
	struct MartyClientParameters myParameters;

	struct MartyMessageHandle *deliveryTracker[MAX_MESSAGE_ID];
	pthread_mutex_t deliveryLock;
};

char *alloc_marty_version_string(const int instanceVersion[3], const char *instanceName);

int marty_client_data_factory(struct MartyClientData *clientData);
void marty_client_parameters_factory(struct MartyClientParameters *parameters);
void marty_client_endpoint_factory(struct MartyClientEndpoint *endpoint);
void marty_message_handle_factory(struct MartyMessageHandle *handle);
void marty_message_handle_free(struct MartyMessageHandle *handle, bool clearList);

void marty_part_log(char *message, int level);
void marty_full_log(char *message, int level, struct MartyClientData *data);

void marty_message_handler(struct MartyMessageHandle *handle,
	struct MMStatus *status, struct MartyClientData *data);

void marty_net_write(struct MMPacket *txPacket, struct MMStatus *status,
	struct MartyClientData *clientData);

void marty_set_application(struct MMPacket *packet, struct MMStatus *status,
	uint8_t *messageId, mmp_arg argument);

enum MMPError marty_init(struct MartyClientData *clientData);
enum MMPError spawn_marty_socket(struct MartyClientData *clientData);

void *marty_network_loop(void *clientData);
